property spriteNum, getPDLError, myDisplayType, mySprite, myMember, myWidthAdjust, myHeightAdjust, myOffStageLoc

on beginSprite me
  Initialize(me)
end

on endSprite me
  mySprite.visible = 1
end

on Initialize me
  mySprite = sprite(me.spriteNum)
  myMember = mySprite.member
  memberType = myMember.type
  case memberType of
    #field, #text:
    otherwise:
      return ErrorAlert(me, #invalidMemberType, memberType)
  end case
  myDisplayType = symbol(myDisplayType)
  if myMember.type = #field then
    myWidthAdjust = (myMember.margin + myMember.border) * 2
    myHeightAdjust = myMember.margin + (myMember.border * 2)
  else
    myWidthAdjust = 0
    myHeightAdjust = 0
  end if
  myMember.text = EMPTY
  if myDisplayType = #Tooltip then
    myMember.boxType = #fixed
    myOffStageLoc = point(999, 999)
    mySprite.loc = myOffStageLoc
  end if
end

on BestRect me, theString
  myMember.rect = rect(0, 0, 8000, 0)
  myMember.text = theString
  BestRect = myMember.rect
  theLine = the number of lines in theString
  theWidth = 0
  checkedChars = 0
  repeat while theLine
    endOfLine = offset(RETURN, theString)
    if not endOfLine then
      endOfLine = the number of chars in theString + 1
      myMember.text = myMember.text & RETURN
    end if
    checkedChars = checkedChars + endOfLine
    endPoint = charPosToLoc(myMember, checkedChars)
    lineWidth = endPoint[1]
    if lineWidth > theWidth then
      theWidth = lineWidth
    end if
    delete char 1 to endOfLine of theString
    theLine = theLine - 1
  end repeat
  lastChar = myMember.char.count
  lastCharLoc = charPosToLoc(myMember, lastChar)
  theHeight = lastCharLoc[2]
  BestRect[3] = theWidth + 1
  BestRect[4] = theHeight + 1
  return BestRect
end

on GetTopLeft me, theLoc, theAlignment, memberRect
  case theAlignment of
    #bottomCenter:
      return theLoc - [memberRect.width / 2, memberRect.height]
    #bottomRight:
      return theLoc - [memberRect.width, memberRect.height]
    #bottomLeft:
      return theLoc - [0, memberRect.height]
    #center:
      return theLoc - [memberRect.width / 2, memberRect.height / 2]
    #topCenter:
      return theLoc - [memberRect.width / 2, 0]
    #topRight:
      return theLoc - [memberRect.width, 0]
    otherwise:
      return theLoc
  end case
end

on DisplayText_Enroll me, enrollList
  if ilk(enrollList) <> #list then
    return me
  end if
  if not enrollList.count() then
    enrollList.append(me)
  else
  end if
  return enrollList
end

on DisplayText_SetText me, theString, theLoc, theAlignment
  global gTooltip
  if gTooltip = 0 then
    exit
  end if
  if not stringp(theString) then
    ErrorAlert(me, #invalidString, theString)
    theString = string(theString)
  else
    case ilk(theLoc) of
      #void, #point:
      otherwise:
        ErrorAlert(me, #invalidPoint, theLoc)
        theLoc = point(0, 0)
    end case
  end if
  if (theString = EMPTY) and (myDisplayType = #Tooltip) then
    mySprite.loc = myOffStageLoc
  else
    myMember.text = theString
    if myDisplayType = #Tooltip then
      memberRect = BestRect(me, theString)
      myMember.rect = memberRect
    else
      memberRect = myMember.rect
    end if
    memberRect = memberRect + [0, 0, myWidthAdjust, myHeightAdjust]
    if myDisplayType = #Tooltip then
      if ilk(theLoc) <> #point then
        theLoc = point(0, 0)
      end if
      theLoc = GetTopLeft(me, theLoc, theAlignment, memberRect)
      stageWidth = (the activeWindow).rect.right - (the activeWindow).rect.left
      stageHeight = (the activeWindow).rect.bottom - (the activeWindow).rect.top
      maxH = stageWidth - memberRect.width
      maxV = stageHeight - memberRect.height
      theLoc[1] = max(0, min(theLoc[1], maxH))
      theLoc[2] = max(0, min(theLoc[2], maxV))
      theLoc = theLoc + myMember.regPoint
      mySprite.loc = theLoc
    else
      lastChar = theString.char.count
      textHeight = charPosToLoc(myMember, lastChar)[2]
      if textHeight > mySprite.height then
        myMember.boxType = #scroll
      else
        myMember.boxType = #fixed
      end if
    end if
  end if
end

on DisplayText_GetReference me
  return me
end

on ErrorAlert me, theError, data
  case theError of
    #getPDLError:
      alert("Error: This behavior works only with Field and Text members." & RETURN & RETURN & "Hit OK and then delete this behavior from the sprite." & RETURN & RETURN & "For more information on deleting Behaviors, see the Help system.")
      if the optionDown then
        return [#getPDLError: [#comment: "ERROR:   Wrong member type.   Click 'Cancel'." & RETURN & "             Use only with Field and Text members.", #format: #string, #range: [EMPTY], #default: EMPTY]]
      end if
    otherwise:
      case theError of
        #invalidMemberType:
          behaviorName = string(me)
          delete word 1 of behaviorName
          delete char -30001 of behaviorName
          delete char -30001 of behaviorName
          alert("BEHAVIOR ERROR: Frame " & the frame & ", Sprite " & me.spriteNum & RETURN & RETURN & "Behavior " & behaviorName & " only works with Field and Text members." & RETURN & RETURN & "Current member type = #" & data)
          halt()
        #invalidString:
          if the runMode = "Author" then
            alert("BEHAVIOR ERROR: Frame " & the frame & ", Sprite " & me.spriteNum & RETURN & "Behavior " & behaviorName & RETURN & RETURN & "The DisplayText_SetText handler could not treat the following as a string:" & RETURN & RETURN & data)
          end if
        #invalidPoint:
          if the runMode = "Author" then
            alert("BEHAVIOR ERROR: Frame " & the frame & ", Sprite " & me.spriteNum & RETURN & "Behavior " & behaviorName & RETURN & RETURN & "The DisplayText_SetText handler could not treat the following as a point:" & RETURN & RETURN & data)
          end if
      end case
  end case
end

on getPropertyDescriptionList me
  if not (the currentSpriteNum) then
    exit
  end if
  theMember = sprite(the currentSpriteNum).member
  case theMember.type of
    #field, #text:
    otherwise:
      return ErrorAlert(me, #getPDLError)
  end case
  return [#myDisplayType: [#comment: "Display Text sprite behaves as a:", #format: #symbol, #range: ["status bar (fixed size and position)", "tooltip (dynamic size and position)"], #default: "in a fixed position (status bar)"]]
end
